from DisplayNode import *
import random

class PuzzleGrid(DisplayNode):
    def __init__(self):
        DisplayNode.__init__(self)
        self.defaultColor = (128, 128, 128)
        self.lightUpColor = (255, 255, 255)
        self.gridSize = 3
        self.margin = 5
        self.size = 80
        self.width = self.gridSize * (self.size + self.margin)
        self.height = self.gridSize * (self.size + self.margin)
        self.pos = (0, 0)
        self.anchor = (0, 0)
        self.matched_count = 0
        self.grid = [] 
        self.updateVertices()

    def update(self, dt):
        pass

    def randomLightUp(self):
        numLightUp = random.randint(1, self.gridSize * self.gridSize)
        for i in xrange(numLightUp):
            x = random.randint(0, self.gridSize-1)
            y = random.randint(0, self.gridSize-1)
            self.toggleBlock((x,y))

    def updateVertices(self):
        self.grid = [[0 for x in xrange(self.gridSize)] for x in xrange(self.gridSize)] 
        self.verts = ()
        self.colors = ()

        for i in xrange(self.gridSize):
           for j in xrange(self.gridSize):
               self.colors += self.defaultColor * 4
               self.verts += \
                   (j*self.size + self.margin, i*self.size+self.margin,  \
                    j*self.size + self.margin, (i+1)*self.size, \
                    (j+1)*self.size, (i+1)*self.size,\
                    (j+1)*self.size, i*self.size+self.margin) 
       
    def isMatching(self, obj):
        self.matched_count = 0
        for i in xrange(self.gridSize):
            for j in xrange(self.gridSize):
                if self.isBlockOn((i,j)) == obj.isBlockOn((i,j)):
                    #count only the highlighted
                    if self.isBlockOn((i,j)):
                        self.matched_count += 1
                else:
                    return False
        return True
    
    def resetColors(self):
        self.colors = self.defaultColor * (len(self.colors)/3)  

    def isBlockOn(self, (x,y)):
        idx = y * self.gridSize * 4 * 3 + x * 4 * 3
        color = self.colors[idx]
        #print "Block(",x,",",y,"): ", color, self.lightUpColor 
        return color == self.lightUpColor[0]

    def setBlockColor(self, (x,y), color):
        idx = y * self.gridSize * 4*3+ x *4*3
        nlist = self.colors[:idx]
        #print idx
        nlist += color * 4 
        nlist += self.colors[idx+4*3:]
        #print "len(nlist): ", len(nlist), " len(self.colors): ", len(self.colors)
        self.colors = nlist

       
    def toggleBlock(self, (x, y)):
        if(x < 0 or y < 0):
            return
        if self.isBlockOn((x,y)):
            self.setBlockColor((x,y), self.defaultColor)
        else:
            self.setBlockColor((x,y), self.lightUpColor)

    def lightUp(self, (x, y)):
        if(x < 0 or y < 0):
            return
        #print "lightUp: ", x, " ", y
        self.setBlockColor((x,y), self.lightUpColor)

    #returns the box indx for x,y on screen.
    def getBox(self, (x, y)):
        #trivial reject case
        if( x < self.getMinX() or x > self.getMaxX() or y < self.getMinX() or y > self.getMaxY() ):
                return (-1, -1)
        #print "minX(): ", self.getMinX(), " maxX(): ", self.getMaxX(), " minY(): ", self.getMinY(), " maxY(): ", self.getMaxY()
        #print x, y
        #transform the x,y to the node space.
        node_x = x - (self.pos[0] - self.anchor[0] * self.width)
        node_y = y - (self.pos[1] - self.anchor[1] * self.height)
        #print "node_x, node_y: ", node_x, " ", node_y 

        return (int(node_x / (self.size+self.margin)), int(node_y / (self.size+self.margin)))


