import pyglet
from pyglet.window import key
import math
import random
import sys

from DisplayNode import *
from TimerDisplay import *
from PuzzleGrid import *
from GameCharacter import *

class MessageTicker(DisplayNode):
    def __init__(self):
        DisplayNode.__init__(self)
        self.width = 200
        self.height = 250
        self.boxColor = (0, 0, 255)
        self.drawBoundingBox = True
        
    def update(self, dt):
        pass

class GameWindow(object):
    GAME_PAUSED = 0
    GAME_OVER = 1
    GAME_RUNNING = 2
    GAME_ANIMATING = 3
    def __init__(self):
        self.win = pyglet.window.Window(800, 600)
        self.win.push_handlers(self)
        self.fps = pyglet.clock.ClockDisplay()

        self.userGrid = PuzzleGrid()
        self.enemyGrid = PuzzleGrid()
        self.enemyGrid.pos = (800, 0)
        self.enemyGrid.anchor = (1, 0)
        self.enemyGrid.defaultColor = (50, 0, 0)
        self.enemyGrid.lightUpColor = (255, 0, 0)
        self.enemyGrid.resetColors()
        self.enemyGrid.randomLightUp()

        self.animTimer = TimerDisplay()
        self.animTimer.duration = 5
        self.animTimer.pos = (400, self.userGrid.getMaxY() + self.userGrid.height/2)

        self.messageTicker = MessageTicker()
        self.messageTicker.pos = (400, self.userGrid.getMaxY() + self.userGrid.height/2)

        self.timeKeeper = TimerDisplay()
        self.timeKeeper.drawBoundingBox = False 
        self.timeKeeper.duration = 10
        self.timeKeeper.pos = (400, self.userGrid.getMinY() + self.userGrid.height/2)

        self.playerChar = PlayerCharacter()
        self.playerChar.anchor = (0.5, 0)
        self.playerChar.pos = (self.userGrid.getMinX()+self.userGrid.width/2, self.userGrid.getMaxY())
        self.enemyChar = EnemyCharacter()
        self.enemyChar.anchor = (0.5, 0)
        self.enemyChar.pos = (self.enemyGrid.getMinX()+self.enemyGrid.width/2, self.enemyGrid.getMaxY())


        self.gameState = GameWindow.GAME_RUNNING
        
        #reset when timer runs out.
        self.numPatterns = 0
        self.blockCount = 0

        #reset when user dies
        self.enemyCount = 0

        pyglet.clock.schedule_interval(self.update, 1/60.0)
        pyglet.app.run()

    def update(self, dt):
        if not (self.gameState == GameWindow.GAME_RUNNING or self.gameState == GameWindow.GAME_ANIMATING):
            return

        self.messageTicker.update(dt)

        if self.gameState == GameWindow.GAME_ANIMATING and self.animTimer.isDone:
            #we assume we get a callback after the animations have played
            self.timeKeeper.resetTimer()
            self.numPatterns = 0
            self.blockCount = 0
            self.gameState = GameWindow.GAME_RUNNING
        else:
            self.animTimer.update(dt)

        self.timeKeeper.update(dt)
        if self.timeKeeper.isDone and GameWindow.GAME_RUNNING == self.gameState:
            self.userGrid.resetColors();
            self.enemyGrid.resetColors();
            self.animTimer.resetTimer()
            print "Patterns Matched: ", self.numPatterns
            print "Total Blocks: ", self.blockCount
            self.gameState = GameWindow.GAME_ANIMATING
            if self.numPatterns:
                self.playerChar.setBonusDamage(self.blockCount * 1.0/(self.numPatterns*self.userGrid.gridSize))

            self.playerChar.attackWith(self.enemyChar.getAttack())
            if self.playerChar.health <= 0:
                print "Game Over, You died after killing: ", self.enemyCount
                self.gameState = GameWindow.GAME_OVER
            else:
                if self.numPatterns and self.blockCount:
                    self.enemyChar.attackWith(self.playerChar.getAttack())
                if self.enemyChar.health <= 0:
                    self.enemyCount += 1
                    #create a new enemy
                    self.enemyChar = EnemyCharacter()
                    self.enemyChar.anchor = (0.5, 0)
                    self.enemyChar.pos = (self.enemyGrid.getMinX()+self.enemyGrid.width/2, self.enemyGrid.getMaxY())
        elif GameWindow.GAME_RUNNING == self.gameState:
            if self.userGrid.isMatching(self.enemyGrid):
                self.numPatterns += 1
                self.blockCount += self.userGrid.matched_count
                self.enemyGrid.resetColors()
                self.enemyGrid.randomLightUp()
                self.userGrid.resetColors()

    def on_draw(self):
        self.win.clear()
        self.userGrid.draw()
        self.enemyGrid.draw()
        self.timeKeeper.draw()
        self.playerChar.draw()
        self.enemyChar.draw()
        #self.messageTicker.draw()
        if(self.timeKeeper.isDone):
            self.animTimer.draw()
        self.fps.draw()

    def on_mouse_motion(self, x, y, dx, dy):
        #print x, y, dx, dy
        pass

    def on_key_press(self, symbol, modifiers):
        print symbol

    def on_mouse_press(self, x, y, button, modifiers):
        if GameWindow.GAME_RUNNING == self.gameState:
            self.userGrid.toggleBlock(self.userGrid.getBox((x, y)))

def main():
    """ your app starts here
    """
    GameWindow()
